@@ -1,7 +1,7 @@
 # Copyright (C) 2009 David Sauve, Trapeze
 
 __author__ = 'David Sauve'
-__version__ = (1, 1, 0, 'beta')
+__version__ = (1, 1, 1, 'beta')
 
 import time
 import datetime
@@ -17,7 +17,7 @@ from django.core.exceptions import ImproperlyConfigured
 from django.utils.encoding import smart_unicode, force_unicode
 
 from haystack.backends import BaseSearchBackend, BaseSearchQuery, SearchNode, log_query
-from haystack.exceptions import MissingDependency
+from haystack.exceptions import HaystackError, MissingDependency
 from haystack.fields import DateField, DateTimeField, IntegerField, FloatField, BooleanField, MultiValueField
 from haystack.models import SearchResult
 from haystack.utils import get_identifier
@@ -33,9 +33,14 @@ DOCUMENT_CUSTOM_TERM_PREFIX = 'X'
 DOCUMENT_CT_TERM_PREFIX = DOCUMENT_CUSTOM_TERM_PREFIX + 'CONTENTTYPE'
 
 
+class InvalidIndexError(HaystackError):
+    """Raised when an index can not be opened."""
+    pass
+
+
 class XHValueRangeProcessor(xapian.ValueRangeProcessor):
-    def __init__(self, sb):
-        self.sb = sb
+    def __init__(self, backend):
+        self.backend = backend or SearchBackend()
         xapian.ValueRangeProcessor.__init__(self)
     
     def __call__(self, begin, end):
@@ -50,7 +55,7 @@ class XHValueRangeProcessor(xapian.ValueRangeProcessor):
         colon = begin.find(':')
         field_name = begin[:colon]
         begin = begin[colon + 1:len(begin)]
-        for field_dict in self.sb.schema:
+        for field_dict in self.backend.schema:
             if field_dict['field_name'] == field_name:
                 if not begin:
                     if field_dict['type'] == 'text':
@@ -127,7 +132,24 @@ class SearchBackend(BaseSearchBackend):
         if not os.path.exists(settings.HAYSTACK_XAPIAN_PATH):
             os.makedirs(settings.HAYSTACK_XAPIAN_PATH)
         
+        if not os.access(settings.HAYSTACK_XAPIAN_PATH, os.W_OK):
+            raise IOError("The path to your Xapian index '%s' is not writable for the current user/group." % settings.HAYSTACK_XAPIAN_PATH)
+        
         self.language = language
+        self._schema = None
+        self._content_field_name = None
+        
+    @property
+    def schema(self):
+        if not self._schema:
+            self._content_field_name, self._schema = self.build_schema(self.site.all_searchfields())            
+        return self._schema
+
+    @property
+    def content_field_name(self):
+        if not self._content_field_name:
+            self._content_field_name, self._schema = self.build_schema(self.site.all_searchfields())            
+        return self._content_field_name
     
     def update(self, index, iterable):
         """
@@ -248,7 +270,7 @@ class SearchBackend(BaseSearchBackend):
                query_facets=None, narrow_queries=None, spelling_query=None,
                limit_to_registered_models=True, **kwargs):
         """
-        Executes the search as defined in `query_string`.
+        Executes the Xapian::query as defined in `query`.
         
         Required arguments:
             `query` -- Search query to execute
@@ -276,17 +298,7 @@ class SearchBackend(BaseSearchBackend):
                     `queries` -- A list of query facets
             If faceting was not used, the `facets` key will not be present
         
-        If `query_string` is empty, returns no results.
-        
-        Otherwise, loads the available fields from the database meta data schema
-        and sets up prefixes for each one along with a prefix for `django_ct`,
-        used to filter by model, and loads the current stemmer instance.
-        
-        Afterwards, executes the Xapian query parser to create a query from
-        `query_string` that is then passed to a new `enquire` instance.
-        
-        The resulting match set is passed to :method:`_process_results` for
-        further processing prior to returning a dictionary with the results.
+        If `query` is None, returns no results.
         
         If `HAYSTACK_INCLUDE_SPELLING` was enabled in `settings.py`, the
         extra flag `FLAG_SPELLING_CORRECTION` will be passed to the query parser
@@ -584,7 +596,7 @@ class SearchBackend(BaseSearchBackend):
         """
         facet_dict = {}
         
-        # DS_TODO: Improve this algorithm.  Currently, runs in O(N^3), ouch.
+        # DS_TODO: Improve this algorithm.  Currently, runs in O(N^2), ouch.
         for field in field_facets:
             facet_list = {}
             
@@ -729,28 +741,23 @@ class SearchBackend(BaseSearchBackend):
     
     def _database(self, writable=False):
         """
-        Private method that returns a xapian.Database for use and sets up
-        schema and content_field definitions.
-        
+        Private method that returns a xapian.Database for use.
+
         Optional arguments:
             ``writable`` -- Open the database in read/write mode (default=False)
-        
+
         Returns an instance of a xapian.Database or xapian.WritableDatabase
         """
         if writable:
-            self.content_field_name, self.schema = self.build_schema(self.site.all_searchfields())
-            
             database = xapian.WritableDatabase(settings.HAYSTACK_XAPIAN_PATH, xapian.DB_CREATE_OR_OPEN)
-            database.set_metadata('schema', pickle.dumps(self.schema, pickle.HIGHEST_PROTOCOL))
-            database.set_metadata('content', pickle.dumps(self.content_field_name, pickle.HIGHEST_PROTOCOL))
         else:
-            database = xapian.Database(settings.HAYSTACK_XAPIAN_PATH)
-            
-            self.schema = pickle.loads(database.get_metadata('schema'))
-            self.content_field_name = pickle.loads(database.get_metadata('content'))
-        
+            try:
+                database = xapian.Database(settings.HAYSTACK_XAPIAN_PATH)
+            except xapian.DatabaseOpeningError:
+                raise InvalidIndexError(u'Unable to open index at %s' % settings.HAYSTACK_XAPIAN_PATH)
+
         return database
-    
+
     def _value_column(self, field):
         """
         Private method that returns the column value slot in the database
@@ -800,8 +807,6 @@ class SearchQuery(BaseSearchQuery):
         self.backend = backend or SearchBackend()
     
     def build_query(self):
-        # DS_TODO: How does stemming work with this new refactor?
-        
         if not self.query_filter:
             query = xapian.Query('')
         else:
